#ifndef MYSOLVER_HH
#define MYSOLVER_HH

#include <dune/common/bitsetvector.hh>
#include <dune/common/parametertree.hh>

#include <dune/solvers/iterationsteps/multigridstep.hh>
#include <dune/solvers/iterationsteps/truncatedblockgsstep.hh>
#include <dune/solvers/norms/energynorm.hh>
#include <dune/solvers/solvers/loopsolver.hh>
#include <dune/tnnmg/iterationsteps/genericnonlineargs.hh>
#include <dune/solvers/transferoperators/compressedmultigridtransfer.hh>
#include <dune/fufem/assemblers/transferoperatorassembler.hh>
#include <dune/tnnmg/iterationsteps/tnnmgstep.hh>

template <int dim, class VectorType, class OperatorType, class GridType,
          class BlockProblemType>
class MySolver {
private:
  typedef GenericNonlinearGS<BlockProblemType> NonlinearSmootherType;
  typedef TruncatedNonsmoothNewtonMultigrid<BlockProblemType,
                                            NonlinearSmootherType> SolverType;

public:
  MySolver(Dune::ParameterTree parset, int refinements, double solver_tolerance,
           GridType const &grid, Dune::BitSetVector<dim> const &ignoreNodes);

  ~MySolver();

  SolverType *getSolver();

private:
  TruncatedBlockGSStep<OperatorType, VectorType> linearBaseSolverStep;
  EnergyNorm<OperatorType, VectorType> baseEnergyNorm;
  LoopSolver<VectorType> linearBaseSolver;
  TruncatedBlockGSStep<OperatorType, VectorType> linearPresmoother;
  TruncatedBlockGSStep<OperatorType, VectorType> linearPostsmoother;
  MultigridStep<OperatorType, VectorType> linearIterationStep;
  std::vector<CompressedMultigridTransfer<VectorType> *> transferOperators;
  NonlinearSmootherType nonlinearSmoother;
  SolverType *multigridStep;
};

#endif
