#include <octave/oct.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <cassert>

#include <dune/common/shared_ptr.hh>

#include <dune/tnnmg/problem-classes/bisection.hh>

#include <dune/tectonic/octave/duneoctave.hh>
#include <dune/tectonic/samplefunctional.hh>

DEFUN_DLD(duneminimise, args, nargout, "duneminimise(A,b,f,y)\n\
\n\
Make a minimisation step of x -> 1/2<Ax,x> - <b,x> + H(|x|) using DUNE starting at x\n") {
  int const dim = 2;

  assert(args.length() == 4);
  assert(nargout <= 1);

  typedef Dune::SampleFunctional<dim> Functional;

  Functional::SmallMatrix A;
  Dune::octaveToDune<dim>(args(0).matrix_value(), A);

  Functional::SmallVector b;
  Dune::octaveToDune<dim>(args(1).vector_value(), b);

  Dune::shared_ptr<Functional::NonlinearityType const> phi;

  charNDArray bar = args(2).char_array_value();
  switch (bar(0)) {
    case 'S': {
      Dune::NiceFunction const *pre_f = new Dune::SampleFunction<100>();
      Dune::shared_ptr<Dune::NiceFunction const> f(pre_f);
      phi = Dune::make_shared<Functional::NonlinearityType const>(f);
      break;
    }
    case 's': {
      Dune::NiceFunction const *pre_f = new Dune::SampleFunction<2>();
      Dune::shared_ptr<Dune::NiceFunction const> f(pre_f);
      phi = Dune::make_shared<Functional::NonlinearityType const>(f);
      break;
    }
    case 't': {
      Dune::NiceFunction const *pre_f = new Dune::TrivialFunction();
      Dune::shared_ptr<Dune::NiceFunction const> f(pre_f);
      phi = Dune::make_shared<Functional::NonlinearityType const>(f);
      break;
    }
    default:
      assert(false);
  }
  Functional J(A, b, phi);

  ColumnVector start_octave(args(3).vector_value());
  Functional::SmallVector start;
  Dune::octaveToDune<dim>(start_octave, start);

  Bisection const &bisection =
      Bisection(0.0,   // acceptError: Stop if the search interval has
                       //              become smaller than this number
                1.0,   // acceptFactor: ?
                1e-12, // requiredResidual: ?
                true,  // fastQuadratic
                0);    // safety: acceptance factor for inexact minimization
  Dune::minimise(J, start, 1, bisection);

  Dune::duneToOctave<dim>(start, start_octave);
  return octave_value(start_octave);
}
